local lib = LibStub("LibNameplate-1.0");
if(not lib) then error("API.lua for LibNameplate can't find LibNameplate."); return; end

local tonumber = tonumber
local select = select
local math_floor = math.floor
local UnitExists = UnitExists
local UnitIsUnit = UnitIsUnit
local UnitGUID = UnitGUID
local UnitName = UnitName
local pairs = pairs
local table_insert = table.insert
local unpack = unpack
local debugPrint = lib.debugPrint

--"Star", "Circle", "Diamond", "Triangle", "Moon", "Square", "Cross", "Skull"
--~ local raidIconTexture = "Interface\\TargetingFrame\\UI-RaidTargetingIcons"
--			= left							right
--			= ULx	ULy		LLx		LLy		URx		URy		LRx		LRy
-- star		= 0		0		0		0.25	0.25	0		0.25	0.25
-- circle	= 0.25	0		0.25	0.25	0.5		0		0.5		0.25
-- diamond	= 0.5	0		0.5		0.25	0.75	0		0.75	0.25
-- triangle	= 0.75	0		0.75	0.25	1		0		1		0.25
-- moon		= 0		0.25	0		0.5		0.25	0.25	0.25	0.5
-- square 	= 0.25	0.25	0.25	0.5		0.5		0.25	0.5		0.5
-- cross	= 0.5	0.25	0.5		0.5		0.75	0.25	0.75	0.5
-- skull	= 0.75	0.25	0.75	0.5		1		0.25	1		0.5
local raidIconTexCoord = {--from GetTexCoord. input is ULx and ULy (first 2 values).
	[0]		= {
		[0]		= 1, --star 
		[0.25]	= 5, --moon
	},
	[0.25]	= {
		[0]		= 2, --circle 
		[0.25]	= 6, --square
	},
	[0.5]	= {
		[0]		= 3, --star 
		[0.25]	= 7, --cross
	},
	[0.75]	= {
		[0]		= 4, --star 
		[0.25]	= 8, --skull
	},
}


--Support functions for API.
local function reactionByColor(red, green, blue, a)																											
	if red < .01 and blue < .01 and green > .99 then return "FRIENDLY", "NPC" 
	elseif red < .01 and blue > .99 and green < .01 then return "FRIENDLY", "PLAYER"
	elseif red > .99 and blue < .01 and green > .99 then return "NEUTRAL", "NPC"
	elseif red > .99 and blue < .01 and green < .01 then return "HOSTILE", "NPC"
	else return "HOSTILE", "PLAYER" end
end

local colorToClass = {}
local function pctToInt(number) return math_floor((100*number) + 0.5) end
for classname, color in pairs(RAID_CLASS_COLORS) do
	colorToClass["C"..pctToInt(color.r)+pctToInt(color.g)+pctToInt(color.b)] = classname
end

local function threatByColor( region )
	if not region:IsShown() then return "LOW" end
	
	local redCan, greenCan, blueCan, alphaCan = region:GetVertexColor()
	if greenCan > .7 then return "MEDIUM" end
	if redCan > .7 then return "HIGH" end
end


local function combatByColor(r, g, b, a) 
	return (r > .5 and g < .5)
end


local function GetHealthBarColor(frame)
	if frame.aloftData then
		local r, g, b = frame.aloftData.originalHealthBarR, frame.aloftData.originalHealthBarG, frame.aloftData.originalHealthBarB
		return r, g, b
	end
	
	if frame.originalR and frame.originalG and frame.originalB then
		 --dNamePlates changes the color of the healthbar. r7 now saves the original colors. TY Dawn.
		return frame.originalR, frame.originalG, frame.originalB
	end
	
	local bar = lib.health_bar[frame] or lib:GetHealthBar(frame)
	if bar and bar.GetStatusBarColor then
		return bar:GetStatusBarColor()
	end
	return nil
end

--------------------------------------------------------------
local function RemoveHexColor(inputString)					--
-- Remove hex color code from string. 						--
-- Aloft uses hex codes to color name and level regions.	--
--------------------------------------------------------------
	local find = inputString:find("|c")
	if find then
		inputString = inputString:sub(find+10)
	end
	inputString = inputString:gsub("|r", "")
	return inputString
end

--API 

function lib:GetName(frame)
	local frame = self.realPlate[frame] or frame

	local nameRegion = self.name_region[frame] or self:GetNameRegion(frame)
	if nameRegion and nameRegion.GetText then
		return RemoveHexColor( nameRegion:GetText() )
	end

	return nil
end


function lib:GetLevel(frame)
	local frame = self.realPlate[frame] or frame
	local region = self.level_region[frame] or self:GetLevelRegion(frame)
	if region and region.GetText then
		return tonumber( RemoveHexColor( region:GetText() ) )
	end
	return 0
end


function lib:GetScale(frame)
	local frame = self.realPlate[frame] or frame
	if frame.extended then
		frame = frame.extended
	end
	return frame:GetScale()
end

function lib:GetVisibleFrame(frame)
	local frame = self.realPlate[frame] or frame
	if frame.extended then
		frame = frame.extended
	end
	return frame
end

	
function lib:GetReaction(frame)
	local frame = self.realPlate[frame] or frame

	local r,g,b = GetHealthBarColor(frame)
	if r then
		return reactionByColor(r, g, b )
	end

	return nil
end

function lib:GetType(frame)
	local frame = self.realPlate[frame] or frame
	
	local r, g, b = GetHealthBarColor(frame)
	if r then
		return select(2, reactionByColor( r, g, b ) )
	end

	return nil
end



function lib:IsBoss(frame)
	local frame = self.realPlate[frame] or frame

	local region = self.boss_region[frame] or self:GetBossRegion(frame)
	if region and region.IsShown then
		return region:IsShown() and true or false
	end

	return nil
end


--This will return nil if we're not in a PvP zone (like in cities)
function lib:GetClass(frame)
	local frame = self.realPlate[frame] or frame

	local r, g, b = GetHealthBarColor(frame)
	if r then
		return colorToClass["C"..pctToInt(r)+pctToInt(g)+pctToInt(b)] or nil
	end

	return nil
end

function lib:IsElite(frame)
	local frame = self.realPlate[frame] or frame

	local region = self.elite_region[frame] or self:GetEliteRegion(frame)
	if region and region.IsShown then
		return region:IsShown() and true or false
	end
	return nil
end



function lib:GetThreatSituation(frame)
	local frame = self.realPlate[frame] or frame

	local region = self.threat_region[frame] or self:GetThreatRegion(frame)
	if region and region.GetVertexColor then
		return threatByColor(region)
	end

	return nil
end

function lib:IsTarget(frame)
	local frame = self.realPlate[frame] or frame
	return frame:IsShown() and frame:GetAlpha() == 1 and UnitExists("target") or false
end

function lib:GetHealthMax(frame)
	local frame = self.realPlate[frame] or frame

	local bar = self.health_bar[frame] or self:GetHealthBar(frame)
	if bar and bar.GetMinMaxValues then
		local _, max = bar:GetMinMaxValues()
		return tonumber(max or 0)
	end
	return nil
end

function lib:GetHealth(frame)
	local frame = self.realPlate[frame] or frame

	local bar = self.health_bar[frame] or self:GetHealthBar(frame)
	if bar and bar.GetValue then
		return bar:GetValue()
	end
	return nil
end

function lib:GetRaidIcon(frame)
	local frame = self.realPlate[frame] or frame

	local region = self.raidicon_region[frame] or self:GetRaidIconRegion(frame)
	if region and region.IsShown and region:IsShown() and region.GetTexCoord then
		local ULx,ULy = region:GetTexCoord() --,LLx,LLy,URx,URy,LRx,LRy
		
		if ULx and ULy then
			return raidIconTexCoord[ULx] and raidIconTexCoord[ULx][ULy] or 0
		end
	end
	
	return nil
end

function lib:IsMouseover(frame)
	local frame = self.realPlate[frame] or frame

	local region = self.hightlight_region[frame] or self:GetHightlightRegion(frame)
	if region and region.IsShown then
		return region:IsShown() and true or false
	end
	
	return nil
end


function lib:IsCasting(frame)
	local frame = self.realPlate[frame] or frame

	local bar = self.cast_bar[frame] or self:GetCastBar(frame)
	if bar and bar.IsShown then
		return bar:IsShown() and true or false
	end
	return nil
end


function lib:IsInCombat(frame)
	local frame = self.realPlate[frame] or frame

	local region = self.name_region[frame] or self:GetNameRegion(frame)
	if region and region.GetTextColor then
		return combatByColor( region:GetTextColor() ) and true or false
	end
	
	return nil
end

function lib:IsMarked(frame)
	local frame = self.realPlate[frame] or frame

	local region = self.raidicon_region[frame] or self:GetRaidIconRegion(frame)
	if region and region.IsShown then
		return region:IsShown() and true or false
	end

	return nil
end


function lib:GetGUID(frame)
	local frame = self.realPlate[frame] or frame
	return self.plateGUIDs[frame]
end

-- ***
function lib:GetTargetNameplate()
	if self.targeted and self.targeted:IsShown() then
		return self.fakePlate[self.targeted] or self.targeted
	end
end

function lib:GetNameplateByGUID(GUID)
	if self.GUIDs[GUID] and self.GUIDs[GUID]:IsShown() then
		return self.fakePlate[self.GUIDs[GUID]] or self.GUIDs[GUID]
	end
end


function lib:GetNameplateByName(name)
	if self.names[name] and self.names[name]:IsShown() then
		return self.fakePlate[self.names[name]] or self.names[name]
	end
end

function lib:GetNameplateByUnit(unitID)
	if UnitIsUnit(unitID, "target") then
		return self:GetTargetNameplate()
	end
	local GUID = UnitGUID(unitID)
	if self.GUIDs[GUID] and self.GUIDs[GUID]:IsShown() then
		return self.fakePlate[self.GUIDs[GUID]] or self.GUIDs[GUID]
	end
	local name = UnitName(unitID)
	if self.names[GUID] and self.names[GUID]:IsShown() then
		return self.fakePlate[self.names[GUID]] or self.names[GUID]
	end
end

--Returns all known nameplates. Not just the one's visible.
function lib:GetAllNameplates()
	local frames = {}
	for frame in pairs(self.nameplates) do 
		table_insert(frames, #frames+1, self.fakePlate[frame] or frame)
	end
	return #frames, unpack(frames)
end
